{
  "type": "record",
  "Aspect": {
    "name": "dataHubOAuthSessionKey",
    "keyForEntity": "dataHubOAuthSession",
    "entityCategory": "internal",
    "entityAspects": [
      "dataHubOAuthSessionInfo"
    ]
  },
  "name": "DataHubOAuthSessionKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "fields": [
    {
      "type": "string",
      "name": "id",
      "doc": "Stable UUID generated at session creation."
    }
  ],
  "doc": "Key aspect for a DataHub OAuth2 session entity.\nEach session represents a single OAuth2 login (authorization code grant)."
}