{
  "type": "record",
  "Aspect": {
    "name": "formNotifications"
  },
  "name": "FormNotifications",
  "namespace": "com.linkedin.pegasus2avro.notification",
  "fields": [
    {
      "Searchable": {
        "/*/formUrn": {
          "fieldName": "notificationFormUrn",
          "fieldType": "URN",
          "queryByDefault": false
        }
      },
      "type": {
        "type": "array",
        "items": {
          "type": "record",
          "name": "FormNotificationDetails",
          "namespace": "com.linkedin.pegasus2avro.notification",
          "fields": [
            {
              "java": {
                "class": "com.linkedin.pegasus2avro.common.urn.Urn"
              },
              "type": "string",
              "name": "formUrn",
              "doc": "The urn of the form that these notification details are on behalf of",
              "Urn": "Urn"
            },
            {
              "type": {
                "type": "array",
                "items": {
                  "type": "record",
                  "name": "FormNotificationEntry",
                  "namespace": "com.linkedin.pegasus2avro.notification",
                  "fields": [
                    {
                      "type": "long",
                      "name": "time",
                      "doc": "The time at which this notification was sent in milliseconds"
                    },
                    {
                      "type": "string",
                      "name": "notificationType",
                      "doc": "The notification scenario type for this notification record"
                    }
                  ],
                  "doc": "Info about a specific form notification"
                }
              },
              "name": "notificationLog",
              "default": [],
              "doc": "The list of details per notification sent on behalf of this form"
            }
          ],
          "doc": "Info about all of the notifications a user has received for a given form"
        }
      },
      "name": "notificationDetails",
      "default": [],
      "doc": "List of details for notifications for each form"
    }
  ],
  "doc": "The history of notifications sent to this actor on behalf of forms"
}