# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import EvalConditionClass
from .....schema_classes import EvalConditionResultClass
from .....schema_classes import EvalConditionTypeClass
from .....schema_classes import EvalDocumentReferencesClass
from .....schema_classes import EvalExecutorClass
from .....schema_classes import EvalExecutorTypeClass
from .....schema_classes import EvalInfoClass
from .....schema_classes import EvalLlmJudgeConditionClass
from .....schema_classes import EvalOriginClass
from .....schema_classes import EvalResultClass
from .....schema_classes import EvalResultTypeClass
from .....schema_classes import EvalRunEventClass
from .....schema_classes import EvalRunModeClass
from .....schema_classes import EvalRunStatusClass
from .....schema_classes import EvalRunSummaryClass
from .....schema_classes import EvalStatusClass


EvalCondition = EvalConditionClass
EvalConditionResult = EvalConditionResultClass
EvalConditionType = EvalConditionTypeClass
EvalDocumentReferences = EvalDocumentReferencesClass
EvalExecutor = EvalExecutorClass
EvalExecutorType = EvalExecutorTypeClass
EvalInfo = EvalInfoClass
EvalLlmJudgeCondition = EvalLlmJudgeConditionClass
EvalOrigin = EvalOriginClass
EvalResult = EvalResultClass
EvalResultType = EvalResultTypeClass
EvalRunEvent = EvalRunEventClass
EvalRunMode = EvalRunModeClass
EvalRunStatus = EvalRunStatusClass
EvalRunSummary = EvalRunSummaryClass
EvalStatus = EvalStatusClass

# fmt: on
