{
  "type": "record",
  "Aspect": {
    "name": "dataHubTaskKey",
    "keyForEntity": "dataHubTask",
    "entityCategory": "core",
    "entityAspects": [
      "dataHubTaskInfo",
      "ownership",
      "status"
    ],
    "entityDoc": "A scheduled or event-triggered task definition. Supports agent tasks and imperative tasks."
  },
  "name": "DataHubTaskKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "fields": [
    {
      "Searchable": {},
      "type": "string",
      "name": "id",
      "doc": "Unique identifier for the task definition"
    }
  ],
  "doc": "Key for a DataHub Task definition"
}