# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import AnomalyReviewStateClass
from .....schema_classes import AnomalySourceClass
from .....schema_classes import AnomalySourcePropertiesClass
from .....schema_classes import AnomalySourceTypeClass
from .....schema_classes import MonitorAnomalyEventClass
from .....schema_classes import OperationRangeClass


AnomalyReviewState = AnomalyReviewStateClass
AnomalySource = AnomalySourceClass
AnomalySourceProperties = AnomalySourcePropertiesClass
AnomalySourceType = AnomalySourceTypeClass
MonitorAnomalyEvent = MonitorAnomalyEventClass
OperationRange = OperationRangeClass

# fmt: on
