{
  "type": "record",
  "Aspect": {
    "name": "siblings",
    "schemaVersion": 3
  },
  "name": "Siblings",
  "namespace": "com.linkedin.pegasus2avro.common",
  "fields": [
    {
      "Relationship": {
        "/*": {
          "entityTypes": [
            "dataset",
            "corpuser"
          ],
          "name": "SiblingOf"
        }
      },
      "Searchable": {
        "/*": {
          "addHasValuesToFilters": true,
          "fieldName": "siblings",
          "fieldType": "URN",
          "hasValuesFieldName": "hasSiblings",
          "queryByDefault": false
        }
      },
      "type": {
        "type": "array",
        "items": "string"
      },
      "name": "siblings",
      "doc": "List of sibling entities",
      "Urn": "Urn",
      "urn_is_array": true
    },
    {
      "Searchable": {
        "fieldName": "siblingsPrimary",
        "fieldType": "BOOLEAN"
      },
      "type": "boolean",
      "name": "primary",
      "doc": "If this is the leader entity of the set of siblings"
    },
    {
      "type": [
        "null",
        {
          "type": "record",
          "name": "MetadataAttribution",
          "namespace": "com.linkedin.pegasus2avro.common",
          "fields": [
            {
              "type": "long",
              "name": "time",
              "doc": "When this metadata was updated."
            },
            {
              "java": {
                "class": "com.linkedin.pegasus2avro.common.urn.Urn"
              },
              "type": "string",
              "name": "actor",
              "doc": "The entity (e.g. a member URN) responsible for applying the assocated metadata. This can\neither be a user (in case of UI edits) or the datahub system for automation.",
              "Urn": "Urn"
            },
            {
              "java": {
                "class": "com.linkedin.pegasus2avro.common.urn.Urn"
              },
              "type": [
                "null",
                "string"
              ],
              "name": "source",
              "default": null,
              "doc": "The DataHub source responsible for applying the associated metadata. This will only be filled out\nwhen a DataHub source is responsible. This includes the specific metadata test urn, the automation urn.",
              "Urn": "Urn"
            },
            {
              "type": {
                "type": "map",
                "values": "string"
              },
              "name": "sourceDetail",
              "default": {},
              "doc": "The details associated with why this metadata was applied. For example, this could include\nthe actual regex rule, sql statement, ingestion pipeline ID, etc.\nAlso can include flags like 'propagated'=true or 'inferred'=true."
            }
          ],
          "doc": "Information about who, why, and how this metadata was applied"
        }
      ],
      "name": "attribution",
      "default": null,
      "doc": "Attribution for who set this sibling assignment and how.\nWhen actor is a human user (source is absent), the automated identity\nresolution script will not override this assignment."
    }
  ],
  "doc": "Siblings information of an entity."
}