{
  "type": "record",
  "Aspect": {
    "name": "aiAgentKey",
    "keyForEntity": "aiAgent",
    "entityCategory": "core",
    "entityAspects": [
      "aiAgentInfo",
      "ownership",
      "status"
    ],
    "entityDoc": "An AI agent with custom instructions, tools, and scoping. Supports native (DataHub-managed), system (bootstrapped), and external (cataloged) agents."
  },
  "name": "AIAgentKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "fields": [
    {
      "Searchable": {},
      "type": "string",
      "name": "id",
      "doc": "Unique identifier for the agent, e.g. \"ask-datahub\" or a generated UUID"
    }
  ],
  "doc": "Key for an AI Agent entity"
}