# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import DataHubAgentTaskDefinitionClass
from .....schema_classes import DataHubEvalTaskDefinitionClass
from .....schema_classes import DataHubTaskEventFilterClass
from .....schema_classes import DataHubTaskEventTriggerClass
from .....schema_classes import DataHubTaskEventTypeClass
from .....schema_classes import DataHubTaskInfoClass
from .....schema_classes import DataHubTaskScheduleClass
from .....schema_classes import DataHubTaskSourceClass
from .....schema_classes import DataHubTaskSourceTypeClass
from .....schema_classes import DataHubTaskStateClass
from .....schema_classes import DataHubTaskTriggerClass
from .....schema_classes import DataHubTaskTypeClass


DataHubAgentTaskDefinition = DataHubAgentTaskDefinitionClass
DataHubEvalTaskDefinition = DataHubEvalTaskDefinitionClass
DataHubTaskEventFilter = DataHubTaskEventFilterClass
DataHubTaskEventTrigger = DataHubTaskEventTriggerClass
DataHubTaskEventType = DataHubTaskEventTypeClass
DataHubTaskInfo = DataHubTaskInfoClass
DataHubTaskSchedule = DataHubTaskScheduleClass
DataHubTaskSource = DataHubTaskSourceClass
DataHubTaskSourceType = DataHubTaskSourceTypeClass
DataHubTaskState = DataHubTaskStateClass
DataHubTaskTrigger = DataHubTaskTriggerClass
DataHubTaskType = DataHubTaskTypeClass

# fmt: on
