# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from ......schema_classes import AiAssistantSettingsClass
from ......schema_classes import AiContextSettingsClass
from ......schema_classes import AiInstructionClass
from ......schema_classes import AiInstructionStateClass
from ......schema_classes import AiInstructionTypeClass
from ......schema_classes import AiPluginAuthTypeClass
from ......schema_classes import AiPluginConfigClass
from ......schema_classes import AiPluginSettingsClass
from ......schema_classes import AiPluginTypeClass
from ......schema_classes import ApplicationsSettingsClass
from ......schema_classes import AuthInjectionLocationClass
from ......schema_classes import ContextEvaluationSettingsClass
from ......schema_classes import ContextTaskFrequencyClass
from ......schema_classes import ContextTaskScheduleClass
from ......schema_classes import DocPropagationFeatureSettingsClass
from ......schema_classes import DocumentationAiSettingsClass
from ......schema_classes import EmailIntegrationSettingsClass
from ......schema_classes import GlobalHomePageSettingsClass
from ......schema_classes import GlobalIncidentsSettingsClass
from ......schema_classes import GlobalIntegrationSettingsClass
from ......schema_classes import GlobalNotificationSettingsClass
from ......schema_classes import GlobalSettingsInfoClass
from ......schema_classes import GlobalViewsSettingsClass
from ......schema_classes import GlobalVisualSettingsClass
from ......schema_classes import HelpLinkClass
from ......schema_classes import MaintenanceSeverityClass
from ......schema_classes import MaintenanceWindowSettingsClass
from ......schema_classes import OAuthAiPluginConfigClass
from ......schema_classes import OAuthProviderClass
from ......schema_classes import OAuthSettingsClass
from ......schema_classes import OidcSettingsClass
from ......schema_classes import SampleDataSettingsClass
from ......schema_classes import SampleDataStatusClass
from ......schema_classes import SharedApiKeyAiPluginConfigClass
from ......schema_classes import SlackIntegrationSettingsClass
from ......schema_classes import SlackUserClass
from ......schema_classes import SsoSettingsClass
from ......schema_classes import TeamsChannelClass
from ......schema_classes import TeamsIntegrationSettingsClass
from ......schema_classes import TeamsUserClass
from ......schema_classes import UserApiKeyAiPluginConfigClass


AiAssistantSettings = AiAssistantSettingsClass
AiContextSettings = AiContextSettingsClass
AiInstruction = AiInstructionClass
AiInstructionState = AiInstructionStateClass
AiInstructionType = AiInstructionTypeClass
AiPluginAuthType = AiPluginAuthTypeClass
AiPluginConfig = AiPluginConfigClass
AiPluginSettings = AiPluginSettingsClass
AiPluginType = AiPluginTypeClass
ApplicationsSettings = ApplicationsSettingsClass
AuthInjectionLocation = AuthInjectionLocationClass
ContextEvaluationSettings = ContextEvaluationSettingsClass
ContextTaskFrequency = ContextTaskFrequencyClass
ContextTaskSchedule = ContextTaskScheduleClass
DocPropagationFeatureSettings = DocPropagationFeatureSettingsClass
DocumentationAiSettings = DocumentationAiSettingsClass
EmailIntegrationSettings = EmailIntegrationSettingsClass
GlobalHomePageSettings = GlobalHomePageSettingsClass
GlobalIncidentsSettings = GlobalIncidentsSettingsClass
GlobalIntegrationSettings = GlobalIntegrationSettingsClass
GlobalNotificationSettings = GlobalNotificationSettingsClass
GlobalSettingsInfo = GlobalSettingsInfoClass
GlobalViewsSettings = GlobalViewsSettingsClass
GlobalVisualSettings = GlobalVisualSettingsClass
HelpLink = HelpLinkClass
MaintenanceSeverity = MaintenanceSeverityClass
MaintenanceWindowSettings = MaintenanceWindowSettingsClass
OAuthAiPluginConfig = OAuthAiPluginConfigClass
OAuthProvider = OAuthProviderClass
OAuthSettings = OAuthSettingsClass
OidcSettings = OidcSettingsClass
SampleDataSettings = SampleDataSettingsClass
SampleDataStatus = SampleDataStatusClass
SharedApiKeyAiPluginConfig = SharedApiKeyAiPluginConfigClass
SlackIntegrationSettings = SlackIntegrationSettingsClass
SlackUser = SlackUserClass
SsoSettings = SsoSettingsClass
TeamsChannel = TeamsChannelClass
TeamsIntegrationSettings = TeamsIntegrationSettingsClass
TeamsUser = TeamsUserClass
UserApiKeyAiPluginConfig = UserApiKeyAiPluginConfigClass

# fmt: on
