{
  "type": "record",
  "Aspect": {
    "name": "evalRunSummary"
  },
  "name": "EvalRunSummary",
  "namespace": "com.linkedin.pegasus2avro.eval",
  "fields": [
    {
      "Searchable": {
        "fieldName": "lastFailedAtMillis",
        "fieldType": "DATETIME"
      },
      "type": [
        "null",
        "long"
      ],
      "name": "lastFailedAtMillis",
      "default": null,
      "doc": "Timestamp of the last failing run (epoch ms UTC)."
    },
    {
      "Searchable": {
        "fieldName": "lastErroredAtMillis",
        "fieldType": "DATETIME"
      },
      "type": [
        "null",
        "long"
      ],
      "name": "lastErroredAtMillis",
      "default": null,
      "doc": "Timestamp of the last errored run (epoch ms UTC)."
    },
    {
      "Searchable": {
        "fieldName": "lastPassedAtMillis",
        "fieldType": "DATETIME"
      },
      "type": [
        "null",
        "long"
      ],
      "name": "lastPassedAtMillis",
      "default": null,
      "doc": "Timestamp of the last passing run (epoch ms UTC)."
    },
    {
      "Searchable": {
        "addToFilters": true,
        "fieldName": "evalStatus",
        "fieldType": "KEYWORD"
      },
      "type": [
        "null",
        {
          "type": "enum",
          "name": "EvalStatus",
          "namespace": "com.linkedin.pegasus2avro.eval",
          "symbols": [
            "PASSING",
            "FAILING",
            "ERROR",
            "INIT"
          ],
          "doc": "Derived latest status for an eval, computed from run events."
        }
      ],
      "name": "evalStatus",
      "default": null,
      "doc": "Current derived status."
    }
  ],
  "doc": "Derived aspect tracking the latest eval run status.\nEnables fast UI lookups without scanning the full timeseries."
}