{
  "type": "record",
  "Aspect": {
    "name": "evalKey",
    "keyForEntity": "eval",
    "entityCategory": "core",
    "entityAspects": [
      "evalInfo",
      "evalRunEvent",
      "evalRunSummary",
      "evalDocumentReferences",
      "ownership",
      "status"
    ],
    "entityDoc": "An eval definition \u2014 a golden question with conditions used to verify agent quality. Scoped to an agent or context hub domain."
  },
  "name": "EvalKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "fields": [
    {
      "Searchable": {},
      "type": "string",
      "name": "id",
      "doc": "Unique identifier for this eval"
    }
  ],
  "doc": "Key for an Eval entity"
}