{
  "type": "record",
  "Aspect": {
    "name": "actionRequestEvals"
  },
  "name": "ActionRequestEvals",
  "namespace": "com.linkedin.pegasus2avro.actionrequest",
  "fields": [
    {
      "Relationship": {
        "/*/evalUrn": {
          "entityTypes": [
            "eval"
          ],
          "name": "ActionRequestEval"
        }
      },
      "Searchable": {
        "/*/evalUrn": {
          "fieldName": "evalUrns",
          "fieldType": "URN",
          "queryByDefault": false
        }
      },
      "type": {
        "type": "array",
        "items": {
          "type": "record",
          "name": "ActionRequestEvalAssignment",
          "namespace": "com.linkedin.pegasus2avro.actionrequest",
          "fields": [
            {
              "java": {
                "class": "com.linkedin.pegasus2avro.common.urn.Urn"
              },
              "type": "string",
              "name": "evalUrn",
              "doc": "The eval entity assigned to this proposal.",
              "Urn": "Urn"
            }
          ],
          "doc": "A single eval assigned to an action request (proposal)."
        }
      },
      "name": "evals",
      "doc": "The evals assigned to this proposal."
    }
  ],
  "doc": "Tracks which evals are assigned to an action request (proposal).\nWritten when a user selects evals to run against a proposal's\nproposed document changes. Can be extended later with per-eval\nrun result summaries."
}