# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import AIAgentInfoClass
from .....schema_classes import AIAgentSourceClass
from .....schema_classes import AIAgentSourceTypeClass
from .....schema_classes import AIAgentToolRefClass
from .....schema_classes import DataHubAgentSettingsClass


AIAgentInfo = AIAgentInfoClass
AIAgentSource = AIAgentSourceClass
AIAgentSourceType = AIAgentSourceTypeClass
AIAgentToolRef = AIAgentToolRefClass
DataHubAgentSettings = DataHubAgentSettingsClass

# fmt: on
